@@ -31,47 +31,20 @@
 #include "commonutils.h"
 #include "gdal_utils_priv.h"
 
-CPL_CVSID("$Id$")
-
 /************************************************************************/
 /*                               Usage()                                */
 /************************************************************************/
 
-static void Usage( const char* pszErrorMsg = nullptr )
+static void Usage(const char *pszErrorMsg = nullptr)
 {
-    printf("nearblack [-of format] [-white | [-color c1,c2,c3...cn]*] [-near dist] [-nb non_black_pixels]\n"
-           "          [-setalpha] [-setmask] [-o outfile] [-q] [-co \"NAME=VALUE\"]* infile\n");
+    fprintf(stderr, "%s\n\n", GDALNearblackGetParserUsage().c_str());
 
-    if( pszErrorMsg != nullptr )
+    if (pszErrorMsg != nullptr)
         fprintf(stderr, "\nFAILURE: %s\n", pszErrorMsg);
 
     exit(1);
 }
 
-/************************************************************************/
-/*                       GDALNearblackOptionsForBinaryNew()             */
-/************************************************************************/
-
-static GDALNearblackOptionsForBinary *GDALNearblackOptionsForBinaryNew()
-{
-    return static_cast<GDALNearblackOptionsForBinary *>(
-        CPLCalloc(1, sizeof(GDALNearblackOptionsForBinary)));
-}
-
-/************************************************************************/
-/*                       GDALNearblackOptionsForBinaryFree()            */
-/************************************************************************/
-
-static void GDALNearblackOptionsForBinaryFree(
-    GDALNearblackOptionsForBinary* psOptionsForBinary )
-{
-    if( psOptionsForBinary )
-    {
-        CPLFree(psOptionsForBinary->pszInFile);
-        CPLFree(psOptionsForBinary->pszOutFile);
-        CPLFree(psOptionsForBinary);
-    }
-}
 /************************************************************************/
 /*                                main()                                */
 /************************************************************************/
@@ -79,100 +52,80 @@ static void GDALNearblackOptionsForBinaryFree(
 MAIN_START(argc, argv)
 {
     /* Check strict compilation and runtime library version as we use C++ API */
-    if( !GDAL_CHECK_VERSION(argv[0]) )
+    if (!GDAL_CHECK_VERSION(argv[0]))
         exit(1);
 
     EarlySetConfigOptions(argc, argv);
 
-/* -------------------------------------------------------------------- */
-/*      Generic arg processing.                                         */
-/* -------------------------------------------------------------------- */
+    /* -------------------------------------------------------------------- */
+    /*      Generic arg processing.                                         */
+    /* -------------------------------------------------------------------- */
     GDALAllRegister();
 
-    if( CPLGetConfigOption("GDAL_CACHEMAX", nullptr) == nullptr )
-        GDALSetCacheMax( 100000000 );
+    if (CPLGetConfigOption("GDAL_CACHEMAX", nullptr) == nullptr)
+        GDALSetCacheMax(100000000);
     argc = GDALGeneralCmdLineProcessor(argc, &argv, 0);
-    if( argc < 1 )
-        exit( -argc );
+    if (argc < 1)
+        exit(-argc);
 
-    for( int i = 0; i < argc; i++ )
-    {
-        if( EQUAL(argv[i], "--utility_version") )
-        {
-            printf("%s was compiled against GDAL %s and "
-                   "is running against GDAL %s\n",
-                   argv[0], GDAL_RELEASE_NAME, GDALVersionInfo("RELEASE_NAME"));
-            CSLDestroy(argv);
-            return 0;
-        }
-        else if( EQUAL(argv[i], "--help") )
-        {
-            Usage();
-        }
-    }
-
-    GDALNearblackOptionsForBinary* psOptionsForBinary =
-        GDALNearblackOptionsForBinaryNew();
+    GDALNearblackOptionsForBinary sOptionsForBinary;
     GDALNearblackOptions *psOptions =
-        GDALNearblackOptionsNew(argv + 1, psOptionsForBinary);
+        GDALNearblackOptionsNew(argv + 1, &sOptionsForBinary);
     CSLDestroy(argv);
 
-    if( psOptions == nullptr )
+    if (psOptions == nullptr)
     {
         Usage();
     }
 
-    if( !(psOptionsForBinary->bQuiet) )
+    if (!(sOptionsForBinary.bQuiet))
     {
         GDALNearblackOptionsSetProgress(psOptions, GDALTermProgress, nullptr);
     }
 
-    if( psOptionsForBinary->pszInFile == nullptr )
-        Usage("No input file specified.");
+    if (sOptionsForBinary.osOutFile.empty())
+        sOptionsForBinary.osOutFile = sOptionsForBinary.osInFile;
 
-    if( psOptionsForBinary->pszOutFile == nullptr )
-        psOptionsForBinary->pszOutFile =
-            CPLStrdup(psOptionsForBinary->pszInFile);
-
-/* -------------------------------------------------------------------- */
-/*      Open input file.                                                */
-/* -------------------------------------------------------------------- */
+    /* -------------------------------------------------------------------- */
+    /*      Open input file.                                                */
+    /* -------------------------------------------------------------------- */
     GDALDatasetH hInDS = nullptr;
     GDALDatasetH hOutDS = nullptr;
     bool bCloseRetDS = false;
 
-    if( strcmp(psOptionsForBinary->pszOutFile,
-               psOptionsForBinary->pszInFile) == 0 )
+    if (sOptionsForBinary.osOutFile == sOptionsForBinary.osInFile)
     {
-        hInDS = GDALOpen(psOptionsForBinary->pszInFile, GA_Update);
+        hInDS = GDALOpen(sOptionsForBinary.osInFile.c_str(), GA_Update);
         hOutDS = hInDS;
     }
     else
     {
-        hInDS = GDALOpen(psOptionsForBinary->pszInFile, GA_ReadOnly);
+        hInDS = GDALOpen(sOptionsForBinary.osInFile.c_str(), GA_ReadOnly);
         bCloseRetDS = true;
     }
 
-    if( hInDS == nullptr )
+    if (hInDS == nullptr)
         exit(1);
 
     int bUsageError = FALSE;
-    GDALDatasetH hRetDS = GDALNearblack(psOptionsForBinary->pszOutFile,
-                                        hOutDS,
-                                        hInDS,
-                                        psOptions, &bUsageError);
-    if( bUsageError )
+    GDALDatasetH hRetDS = GDALNearblack(sOptionsForBinary.osOutFile.c_str(),
+                                        hOutDS, hInDS, psOptions, &bUsageError);
+    if (bUsageError)
         Usage();
-    const int nRetCode = hRetDS ? 0 : 1;
+    int nRetCode = hRetDS ? 0 : 1;
 
-    GDALClose(hInDS);
-    if( bCloseRetDS )
-        GDALClose(hRetDS);
+    if (GDALClose(hInDS) != CE_None)
+        nRetCode = 1;
+    if (bCloseRetDS)
+    {
+        if (GDALClose(hRetDS) != CE_None)
+            nRetCode = 1;
+    }
     GDALNearblackOptionsFree(psOptions);
-    GDALNearblackOptionsForBinaryFree(psOptionsForBinary);
 
     GDALDestroyDriverManager();
 
     return nRetCode;
 }
+
 MAIN_END
