@@ -53,22 +53,22 @@ class HFADataset final : public GDALPamDataset
 {
     friend class HFARasterBand;
 
-    HFAHandle hHFA;
+    HFAHandle hHFA = nullptr;
 
-    bool bMetadataDirty;
+    bool bMetadataDirty = false;
 
-    bool bGeoDirty;
+    bool bGeoDirty = false;
     double adfGeoTransform[6];
     OGRSpatialReference m_oSRS{};
 
-    bool bIgnoreUTM;
+    bool bIgnoreUTM = false;
 
     CPLErr ReadProjection();
     CPLErr WriteProjection();
-    bool bForceToPEString;
+    bool bForceToPEString = false;
+    bool bDisablePEString = false;
 
-    int nGCPCount;
-    GDAL_GCP asGCPList[36];
+    std::vector<gdal::GCP> m_aoGCPs{};
 
     void UseXFormStack(int nStepCount, Efga_Polynomial *pasPolyListForward,
                        Efga_Polynomial *pasPolyListReverse);
@@ -113,7 +113,7 @@ class HFADataset final : public GDALPamDataset
     virtual CPLErr SetMetadataItem(const char *, const char *,
                                    const char * = "") override;
 
-    virtual void FlushCache(bool bAtClosing) override;
+    virtual CPLErr FlushCache(bool bAtClosing) override;
     virtual CPLErr IBuildOverviews(const char *pszResampling, int nOverviews,
                                    const int *panOverviewList, int nListBands,
                                    const int *panBandList,
