@@ -30,7 +30,9 @@
 #include "keadataset.h"
 #include "keaband.h"
 #include "keacopy.h"
+#include "keadrivercore.h"
 #include "../frmts/hdf5/hdf5vfl.h"
+#include "cpl_vsi_virtual.h"
 
 /************************************************************************/
 /*                     KEADatasetDriverUnload()                        */
@@ -48,6 +50,8 @@ GDALDataType KEA_to_GDAL_Type(kealib::KEADataType ekeaType)
     switch (ekeaType)
     {
         case kealib::kea_8int:
+            egdalType = GDT_Int8;
+            break;
         case kealib::kea_8uint:
             egdalType = GDT_Byte;
             break;
@@ -88,6 +92,9 @@ kealib::KEADataType GDAL_to_KEA_Type(GDALDataType egdalType)
     kealib::KEADataType ekeaType = kealib::kea_undefined;
     switch (egdalType)
     {
+        case GDT_Int8:
+            ekeaType = kealib::kea_8int;
+            break;
         case GDT_Byte:
             ekeaType = kealib::kea_8uint;
             break;
@@ -125,7 +132,7 @@ kealib::KEADataType GDAL_to_KEA_Type(GDALDataType egdalType)
 // static function - pointer set in driver
 GDALDataset *KEADataset::Open(GDALOpenInfo *poOpenInfo)
 {
-    if (Identify(poOpenInfo))
+    if (KEADriverIdentify(poOpenInfo))
     {
         try
         {
@@ -174,6 +181,14 @@ GDALDataset *KEADataset::Open(GDALOpenInfo *poOpenInfo)
                      poOpenInfo->pszFilename, e.what());
             return nullptr;
         }
+        catch (...)
+        {
+            // was a problem - can't be a valid file
+            CPLError(CE_Failure, CPLE_OpenFailed,
+                     "Attempt to open file `%s' failed. Error: unknown\n",
+                     poOpenInfo->pszFilename);
+            return nullptr;
+        }
     }
     else
     {
@@ -182,34 +197,6 @@ GDALDataset *KEADataset::Open(GDALOpenInfo *poOpenInfo)
     }
 }
 
-// static function- pointer set in driver
-// this function is called in preference to Open
-//
-int KEADataset::Identify(GDALOpenInfo *poOpenInfo)
-{
-
-    /* -------------------------------------------------------------------- */
-    /*      Is it an HDF5 file?                                             */
-    /* -------------------------------------------------------------------- */
-    static const char achSignature[] = "\211HDF\r\n\032\n";
-
-    if (poOpenInfo->pabyHeader == nullptr ||
-        memcmp(poOpenInfo->pabyHeader, achSignature, 8) != 0)
-    {
-        return 0;
-    }
-
-    // avoid using kealib::KEAImageIO::isKEAImage as this is likely
-    // to be too slow over curl etc (and doesn't take a HDF5 file handle
-    // anyway).
-    // Just test the extension
-    CPLString osExt(CPLGetExtension(poOpenInfo->pszFilename));
-    if (EQUAL(osExt, "KEA"))
-        return 1;
-    else
-        return 0;
-}
-
 // static function
 H5::H5File *KEADataset::CreateLL(const char *pszFilename, int nXSize,
                                  int nYSize, int nBandsIn, GDALDataType eType,
@@ -225,6 +212,20 @@ H5::H5File *KEADataset::CreateLL(const char *pszFilename, int nXSize,
             pszFilename);
         return nullptr;
     }
+
+    // This helps avoiding issues with H5File handles in a bad state, that
+    // may cause crashes at process termination
+    // Cf https://github.com/OSGeo/gdal/issues/8743
+    if (VSIFileManager::GetHandler(pszFilename) !=
+        VSIFileManager::GetHandler(""))
+    {
+        CPLError(CE_Failure, CPLE_OpenFailed,
+                 "Attempt to create file `%s' failed. /vsi file systems not "
+                 "supported\n",
+                 pszFilename);
+        return nullptr;
+    }
+
     // process any creation options in papszParamList
     // default value
     unsigned int nimageblockSize = kealib::KEA_IMAGE_CHUNK_SIZE;
@@ -298,6 +299,13 @@ H5::H5File *KEADataset::CreateLL(const char *pszFilename, int nXSize,
                  e.what());
         return nullptr;
     }
+    catch (...)
+    {
+        CPLError(CE_Failure, CPLE_OpenFailed,
+                 "Attempt to create file `%s' failed. Error: unknown\n",
+                 pszFilename);
+        return nullptr;
+    }
 }
 
 // static function- pointer set in driver
