@@ -29,60 +29,39 @@
 #include "ogr_dwg.h"
 #include "cpl_conv.h"
 #include "ogrteigha.h"
-
-CPL_CVSID("$Id$")
+#include "ogrdwgdrivercore.h"
 
 /************************************************************************/
-/*                            OGRDWGDriver()                            */
+/*                          OGRDWGDriverUnload()                        */
 /************************************************************************/
 
-OGRDWGDriver::OGRDWGDriver() : poServices(nullptr)
-
-{
-}
-
-/************************************************************************/
-/*                          ~OGRDWGDriver()                             */
-/************************************************************************/
-
-OGRDWGDriver::~OGRDWGDriver()
-
+static void OGRDWGDriverUnload(GDALDriver *)
 {
+    CPLDebug("DWG", "Driver cleanup");
     OGRTEIGHADeinitialize();
 }
 
-/************************************************************************/
-/*                              GetName()                               */
-/************************************************************************/
-
-const char *OGRDWGDriver::GetName()
-
-{
-    return "DWG";
-}
-
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
 
-OGRDataSource *OGRDWGDriver::Open( const char * pszFilename, int /*bUpdate*/ )
-
+static GDALDataset *OGRDWGDriverOpen(GDALOpenInfo *poOpenInfo)
 {
-    if( !EQUAL(CPLGetExtension(pszFilename),"dwg") )
+    if (!OGRDWGDriverIdentify(poOpenInfo))
         return nullptr;
 
     // Check that this is a real file since the driver doesn't support
     // VSI*L API
     VSIStatBuf sStat;
-    if( VSIStat(pszFilename, &sStat) != 0 )
+    if (VSIStat(poOpenInfo->pszFilename, &sStat) != 0)
         return nullptr;
 
-    if( !OGRTEIGHAInitialize() )
+    if (!OGRTEIGHAInitialize())
         return nullptr;
 
-    OGRDWGDataSource   *poDS = new OGRDWGDataSource();
+    OGRDWGDataSource *poDS = new OGRDWGDataSource();
 
-    if( !poDS->Open( OGRDWGGetServices(), pszFilename ) )
+    if (!poDS->Open(OGRDWGGetServices(), poOpenInfo->pszFilename))
     {
         delete poDS;
         poDS = nullptr;
@@ -91,16 +70,6 @@ OGRDataSource *OGRDWGDriver::Open( const char * pszFilename, int /*bUpdate*/ )
     return poDS;
 }
 
-/************************************************************************/
-/*                           TestCapability()                           */
-/************************************************************************/
-
-int OGRDWGDriver::TestCapability( const char * /*pszCap*/ )
-
-{
-    return FALSE;
-}
-
 /************************************************************************/
 /*                           RegisterOGRDWG()                           */
 /************************************************************************/
@@ -108,13 +77,14 @@ int OGRDWGDriver::TestCapability( const char * /*pszCap*/ )
 void RegisterOGRDWG()
 
 {
-    OGRSFDriver* poDriver = new OGRDWGDriver;
-    poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
-                                "AutoCAD DWG" );
-    poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "dwg" );
-    poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "drivers/vector/dwg.html" );
-    poDriver->SetMetadataItem( GDAL_DCAP_FEATURE_STYLES, "YES" );
-    poDriver->SetMetadataItem( GDAL_DCAP_MULTIPLE_VECTOR_LAYERS, "YES" );
+    if (GDALGetDriverByName(DWG_DRIVER_NAME) != nullptr)
+        return;
 
-    OGRSFDriverRegistrar::GetRegistrar()->RegisterDriver( poDriver );
+    GDALDriver *poDriver = new GDALDriver();
+    OGRDWGDriverSetCommonMetadata(poDriver);
+
+    poDriver->pfnOpen = OGRDWGDriverOpen;
+    poDriver->pfnUnloadDriver = OGRDWGDriverUnload;
+
+    GetGDALDriverManager()->RegisterDriver(poDriver);
 }
