@@ -84,6 +84,7 @@ OGRMemLayer::OGRMemLayer(const char *pszName,
     }
 
     m_oMapFeaturesIter = m_oMapFeatures.begin();
+    m_poFeatureDefn->Seal(/* bSealFields = */ true);
 }
 
 /************************************************************************/
@@ -613,7 +614,8 @@ int OGRMemLayer::TestCapability(const char *pszCap)
 /*                            CreateField()                             */
 /************************************************************************/
 
-OGRErr OGRMemLayer::CreateField(OGRFieldDefn *poField, int /* bApproxOK */)
+OGRErr OGRMemLayer::CreateField(const OGRFieldDefn *poField,
+                                int /* bApproxOK */)
 {
     if (!m_bUpdatable)
         return OGRERR_FAILURE;
@@ -621,12 +623,14 @@ OGRErr OGRMemLayer::CreateField(OGRFieldDefn *poField, int /* bApproxOK */)
     // Simple case, no features exist yet.
     if (m_nFeatureCount == 0)
     {
-        m_poFeatureDefn->AddFieldDefn(poField);
+        whileUnsealing(m_poFeatureDefn)->AddFieldDefn(poField);
         return OGRERR_NONE;
     }
 
     // Add field definition and setup remap definition.
-    m_poFeatureDefn->AddFieldDefn(poField);
+    {
+        whileUnsealing(m_poFeatureDefn)->AddFieldDefn(poField);
+    }
 
     // Remap all the internal features.  Hopefully there aren't any
     // external features referring to our OGRFeatureDefn!
@@ -682,7 +686,7 @@ OGRErr OGRMemLayer::DeleteField(int iField)
 
     m_bUpdated = true;
 
-    return m_poFeatureDefn->DeleteFieldDefn(iField);
+    return whileUnsealing(m_poFeatureDefn)->DeleteFieldDefn(iField);
 }
 
 /************************************************************************/
@@ -712,7 +716,7 @@ OGRErr OGRMemLayer::ReorderFields(int *panMap)
 
     m_bUpdated = true;
 
-    return m_poFeatureDefn->ReorderFieldDefns(panMap);
+    return whileUnsealing(m_poFeatureDefn)->ReorderFieldDefns(panMap);
 }
 
 /************************************************************************/
@@ -732,6 +736,7 @@ OGRErr OGRMemLayer::AlterFieldDefn(int iField, OGRFieldDefn *poNewFieldDefn,
     }
 
     OGRFieldDefn *poFieldDefn = m_poFeatureDefn->GetFieldDefn(iField);
+    auto oTemporaryUnsealer(poFieldDefn->GetTemporaryUnsealer());
 
     if ((nFlagsIn & ALTER_TYPE_FLAG) &&
         (poFieldDefn->GetType() != poNewFieldDefn->GetType() ||
@@ -876,6 +881,7 @@ OGRErr OGRMemLayer::AlterGeomFieldDefn(
     }
 
     auto poFieldDefn = m_poFeatureDefn->GetGeomFieldDefn(iGeomField);
+    auto oTemporaryUnsealer(poFieldDefn->GetTemporaryUnsealer());
 
     if (nFlagsIn & ALTER_GEOM_FIELD_DEFN_NAME_FLAG)
         poFieldDefn->SetName(poNewGeomFieldDefn->GetNameRef());
@@ -931,7 +937,7 @@ OGRErr OGRMemLayer::AlterGeomFieldDefn(
 /*                          CreateGeomField()                           */
 /************************************************************************/
 
-OGRErr OGRMemLayer::CreateGeomField(OGRGeomFieldDefn *poGeomField,
+OGRErr OGRMemLayer::CreateGeomField(const OGRGeomFieldDefn *poGeomField,
                                     int /* bApproxOK */)
 {
     if (!m_bUpdatable)
@@ -940,12 +946,12 @@ OGRErr OGRMemLayer::CreateGeomField(OGRGeomFieldDefn *poGeomField,
     // Simple case, no features exist yet.
     if (m_nFeatureCount == 0)
     {
-        m_poFeatureDefn->AddGeomFieldDefn(poGeomField);
+        whileUnsealing(m_poFeatureDefn)->AddGeomFieldDefn(poGeomField);
         return OGRERR_NONE;
     }
 
     // Add field definition and setup remap definition.
-    m_poFeatureDefn->AddGeomFieldDefn(poGeomField);
+    whileUnsealing(m_poFeatureDefn)->AddGeomFieldDefn(poGeomField);
 
     const int nGeomFieldCount = m_poFeatureDefn->GetGeomFieldCount();
     std::vector<int> anRemap(nGeomFieldCount);
