@@ -193,7 +193,7 @@ bool CPLODBCDriverInstaller::FindMdbToolsDriverLib(CPLString &osDriverFile)
         if (LibraryExists(strLibPath.c_str()))
         {
             // Save custom driver path
-            osDriverFile = strLibPath;
+            osDriverFile = std::move(strLibPath);
             return true;
         }
     }
@@ -278,7 +278,7 @@ bool CPLODBCDriverInstaller::LibraryExists(const char *pszLibPath)
 
 void CPLODBCDriverInstaller::InstallMdbToolsDriver()
 {
-#ifdef WIN32
+#ifdef _WIN32
     return;
 #else
     static std::once_flag oofDriverInstallAttempted;
@@ -304,8 +304,7 @@ void CPLODBCDriverInstaller::InstallMdbToolsDriver()
                 CPLAssert(!osDriverFile.empty());
                 CPLDebug("ODBC", "MDB Tools driver: %s", osDriverFile.c_str());
 
-                CPLString driverName("Microsoft Access Driver (*.mdb)");
-                CPLString driver(driverName);
+                std::string driver("Microsoft Access Driver (*.mdb)");
                 driver += '\0';
                 driver += "Driver=";
                 driver += osDriverFile;  // Found by FindDriverLib()
@@ -614,13 +613,22 @@ bool CPLODBCSession::ConnectToMsAccess(const char *pszName,
     const auto Connect =
         [this, &pszName](const char *l_pszDSNStringTemplate, bool bVerboseError)
     {
-        char *pszDSN = static_cast<char *>(
-            CPLMalloc(strlen(pszName) + strlen(l_pszDSNStringTemplate) + 100));
-        /* coverity[tainted_string] */
-        snprintf(pszDSN, strlen(pszName) + strlen(l_pszDSNStringTemplate) + 100,
-                 l_pszDSNStringTemplate, pszName);
-        CPLDebug("ODBC", "EstablishSession(%s)", pszDSN);
-        int bError = !EstablishSession(pszDSN, nullptr, nullptr);
+        std::string osDSN;
+        constexpr const char *PCT_S = "%s";
+        const char *pszPctS = strstr(l_pszDSNStringTemplate, PCT_S);
+        if (!pszPctS)
+        {
+            osDSN = l_pszDSNStringTemplate;
+        }
+        else
+        {
+            osDSN.assign(l_pszDSNStringTemplate,
+                         pszPctS - l_pszDSNStringTemplate);
+            osDSN += pszName;
+            osDSN += (pszPctS + strlen(PCT_S));
+        }
+        CPLDebug("ODBC", "EstablishSession(%s)", osDSN.c_str());
+        int bError = !EstablishSession(osDSN.c_str(), nullptr, nullptr);
         if (bError)
         {
             if (bVerboseError)
@@ -628,13 +636,11 @@ bool CPLODBCSession::ConnectToMsAccess(const char *pszName,
                 CPLError(CE_Failure, CPLE_AppDefined,
                          "Unable to initialize ODBC connection to DSN for %s,\n"
                          "%s",
-                         pszDSN, GetLastError());
+                         osDSN.c_str(), GetLastError());
             }
-            CPLFree(pszDSN);
             return false;
         }
 
-        CPLFree(pszDSN);
         return true;
     };
 
@@ -1600,6 +1606,7 @@ int CPLODBCStatement::Failed(int nResultCode)
 
     return TRUE;
 }
+
 //! @endcond
 
 /************************************************************************/
@@ -1638,6 +1645,24 @@ void CPLODBCStatement::Append(const char *pszText)
     m_nStatementLen += nTextLen;
 }
 
+/************************************************************************/
+/*                      Append(const std::string &)                     */
+/************************************************************************/
+
+/**
+ * Append text to internal command.
+ *
+ * The passed text is appended to the internal SQL command text.
+ *
+ * @param s text to append.
+ */
+
+void CPLODBCStatement::Append(const std::string &s)
+
+{
+    Append(s.c_str());
+}
+
 /************************************************************************/
 /*                     AppendEscaped(const char *)                      */
 /************************************************************************/
